//+------------------------------------------------------------------+
//|                                                    vadym1989.mq4 |
//+------------------------------------------------------------------+


// Trend Detection function
#define BULL 1
#define BEAR 2
#define CloseBULL 3
#define CloseBEAR 4
 
// Input variables

extern int     TradeBar      = 1;
extern int     MAPeriod      = 200;
extern int     MAMethod      = 3; // 0-SMA, 1-EMA, 2-SMMA, 3-LWMA
extern int     MAPrice       = 0; // 0-Close, 1-Open, 2-High, 3-Low, 4-Median, 5-Typical, 6-Weighted
extern int     MAShift       = 0;

extern int     MACDFastEMA   = 25;
extern int     MACDSlowEMA   = 52;
extern int     MACDSMA       = 1;
extern int     MACDPrice     = 0; // 0-Close, 1-Open, 2-High, 3-Low, 4-Median, 5-Typical, 6-Weighted

extern bool    CloseOnRev    = true;

extern double  Lots          = 0.01;
extern bool    MoneyM        = true;
extern int     Risk          = 1;

extern int     StopLoss      = 0;
extern int     TakeProfit    = 0;
extern int     Slippage      = 3;

extern bool    StopTrailing  = true;
extern int     MinProfit     = 0;
extern int     TrailingStop  = 30;
extern int     TrailingStep  = 1;

extern string  CommentOrder  = "vadym1989";
 
//--------------------------------------// Global Variables //----------------------------------//

int Total_Open_Orders = 1; // we only want to open one order at a time and only manage the one order
int cnt = 0;               // counter variable, used in for() loops
bool init_variables;       // init variable when program starts
datetime PreviousBar;      // record the candle/bar time
int LastTrendDirection;    // record the previous trends direction
int FileHandle;            // variable for file handling
int Magic;         

//////////////////////////////////////////////////////////////////////////////////////////////////
//+--------------------------------------------------------------------------------------------+//
//|                                 Expert Initialization Function                             |//
//+--------------------------------------------------------------------------------------------+//
////////////////////////////////////////////////////////////////////////////////////////////////// 

int init()
  {
   Magic = AccountBalance()+13131313;
   if(IsTesting()==false)
   {
      init_variables = true;  // Allows us to init variables in the start function because 
                              // we cannot do this in the init function because the variables 
                              // use values from the chart 
                           
//----------------------------------------------------------------------------------------------//
//---------// create data file if it does not exist and load variables if file does exist //----//

      FileHandle=FileOpen("vadym1989.dat",FILE_BIN | FILE_READ);
      if(FileHandle<1)
      {
         Print("vadym1989.dat not found.");
         FileHandle=FileOpen("vadym1989.dat",FILE_BIN | FILE_WRITE);
         if(FileHandle>0)
         {
            LastTrendDirection=0;
            FileWriteInteger(FileHandle,LastTrendDirection,SHORT_VALUE);
            FileClose(FileHandle);
            Print("vadym1989.dat has been successfully created.");
         }
         else
         {
            FileClose(FileHandle);
            Print("Failed to create vadym1989.dat file.");
         }  
      }
      else
      {
          LastTrendDirection=FileReadInteger(FileHandle,SHORT_VALUE);
          Print("vadym1989 Variables loaded from file.");
          FileClose(FileHandle);
      }
   }      
   return(0);
  }
//----------------------------------------------------------------------------------------------// 
  
int deinit()
  {
   return(0);
  }
//----------------------------------------------------------------------------------------------//
  
int start()

  {if(StopTrailing) TrailPositions();
   {
      Lots = NormalizeDouble(Lots,2);
      if(Lots < 0.01)
      {
         Comment("The variable Lots must be 0.01 or greater to open an order. ");
         return(0);
      }
   }
   if(MarketInfo(Symbol(),MODE_MINLOT) == 0.1)
   {
      Lots = NormalizeDouble(Lots,1);
      if(Lots < 0.1)
      {
         Comment("The variable Lots must be 0.1 or greater to open an order. ");
         return(0);
      }
   }
   if(MarketInfo(Symbol(),MODE_MINLOT) == 1)
   {
      Lots = NormalizeDouble(Lots,0);
      if(Lots < 1)
      {
         Comment("The variable Lots must be 1 or greater to open an order. ");
         return(0);
      }
   }
         
   // init variables when the expert advisor first starts running
   if(init_variables == true)
   {
      PreviousBar = Time[0];     // record the current canle/bar open time
      
      // place code here that you only wnat to run one time
       
      init_variables = false;    // change to false so we only init variable once
   }
   
//---------------------------------// Record Trends Direction //--------------------------------//

   if(LastTrendDirection==0)
      {
         if(TrendDetection()==BULL)
         {
            LastTrendDirection=BULL;
         }
         if(TrendDetection()==BEAR)
         {
            LastTrendDirection=BEAR;
         }
         // save variables to file
         SaveVariables();
      }
//---------// Money Management and perform analysis to open orders on new candle/bar //---------//

   if(MoneyM == true)
   Lots = MathCeil(AccountFreeMargin()/1000*Risk)/10;
   else Lots = Lots;
   if(NewBar() == true)
   {
   
//---------// only perform analysis and close order if we only have one order open //-----------//
      if(TotalOpenOrders() == Total_Open_Orders && SelectTheOrder() == true)
      {
         if(OrderType() == OP_BUY && CloseOnRev==true && TrendDetection() == CloseBULL) 
         {
            OrderClose(OrderTicket(),OrderLots(),Bid,Slippage,Green);
         }
         if(OrderType() == OP_SELL && CloseOnRev==true && TrendDetection() == CloseBEAR)
         {
            OrderClose(OrderTicket(),OrderLots(),Ask,Slippage,Red);
         }
      }
      
//-- only perform analysis and open new order if we have not reached our Total_Open_Orders max -//

      if(TotalOpenOrders() < Total_Open_Orders)
      {
//--------------------------------------// Open BUY //------------------------------------------//
         if(TrendDetection() == BULL)
         {
            if(StopLoss>0 && TakeProfit>0)
            {
               // open order
               OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,Ask-(StopLoss*Point),Ask+(TakeProfit*Point),CommentOrder,Magic,0,Green);
            }
            if(StopLoss>0 && TakeProfit==0)
            {
               // open order
               OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,Ask-(StopLoss*Point),0,CommentOrder,Magic,0,Green);
            }
            if(StopLoss==0 && TakeProfit>0)
            {
               // open order
               OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,0,Ask+(TakeProfit*Point),CommentOrder,Magic,0,Green);
            }
            if(StopLoss==0 && TakeProfit==0)
            {
               // open order
               OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,0,0,CommentOrder,Magic,0,Green);
            }
            LastTrendDirection=BULL;
            // save variables to file
            SaveVariables();
         }
         
//-------------------------------------// Open Sell //------------------------------------------//
         if(TrendDetection() == BEAR)
         {
            if(StopLoss>0 && TakeProfit>0)
            {
               // open order
               OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,Bid+(StopLoss*Point),Bid-(TakeProfit*Point),CommentOrder,Magic,0,Red);
            }
            if(StopLoss>0 && TakeProfit==0)
            {
               // open order
               OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,Bid+(StopLoss*Point),0,CommentOrder,Magic,0,Red);
            }
            if(StopLoss==0 && TakeProfit>0)
            {
               // open order
               OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,0,Bid-(TakeProfit*Point),CommentOrder,Magic,0,Red);
            }
            if(StopLoss==0 && TakeProfit==0)
            {
               // open order
               OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,0,0,CommentOrder,Magic,0,Red);
            }
            LastTrendDirection=BEAR;
            // save variables to file
            SaveVariables();
         }
      }
      
// when back testing only display chart info every candle/bar so we do not slow down back tests //
      if(IsTesting() == true)
      {
         Display_Info();
      }
   }
      
   // when not back testing display chart info every tick
   if(IsTesting() == false)
   {
      Display_Info();
   }
   
   return(0);
  }
 
//-----------------------------------// Common Functions //-------------------------------------//

int SaveVariables()
{
   if(IsTesting()==false)
   {
      //save variables to file
      FileHandle=FileOpen("vadym1989.dat",FILE_BIN | FILE_WRITE);
      if(FileHandle<1)
      {
         Print("vadym1989.dat not found.");
         FileHandle=FileOpen("vadym1989.dat",FILE_BIN | FILE_WRITE);
         if(FileHandle>0)
         {
            FileWriteInteger(FileHandle,LastTrendDirection,SHORT_VALUE);
            FileClose(FileHandle);
            Print("vadym1989.dat has been successfully created.");
         }  
         else
         {
            FileClose(FileHandle);
            Print("Failed to create vadym1989.dat file.");
         }
      }
      else
      {
         FileWriteInteger(FileHandle,LastTrendDirection,SHORT_VALUE);
         FileClose(FileHandle);
         Print("vadym1989 variables successfully saved to file.");
      }  
   }
return(0);
}
 
//------// This function returns the total amount of orders the expert advisor has open //------//

int TotalOpenOrders()
{
   cnt=OrdersTotal();
   int TotalOpenOrders = 0;
   
   if(cnt==0)
   {
      return(0);
   }
   else
   {
      for(;cnt>=0;cnt--)
      {
         RefreshRates();
         OrderSelect(cnt,SELECT_BY_POS);
         if(OrderMagicNumber()==Magic)
         {
            TotalOpenOrders++;
         }
      }
   }
   return(TotalOpenOrders);
}
 
// This function finds the open order and selects it //-----------------------------------------------
int SelectTheOrder()
{
   cnt=OrdersTotal();
      
   if(cnt==0)
   {
      return(false);
   }
   else
   {
      for(;cnt>=0;cnt--)
      {
         RefreshRates();
         OrderSelect(cnt,SELECT_BY_POS);
         if(OrderMagicNumber()==Magic)
         {
            return(true);
         }
      }
   }
   return(false);
}
 
//-----// This function return the value true if the current bar/candle was just formed //------//

bool NewBar()
{
   if(PreviousBar<Time[0])
   {
      PreviousBar = Time[0];
      return(true);
   }
   else
   {
      return(false);
   }
   return(false);    // in case if - else statement is not executed
}

//--------------------------------------// Trend Detection //-----------------------------------//

int TrendDetection()
{ 
//-------------------------- 
   double ma0=iMA(NULL,0,1,0,0,0,TradeBar);
   double ma1=iMA(NULL,0,MAPeriod,0,MAMethod,MAPrice,TradeBar+MAShift);
   double macd=iMACD(NULL,0,MACDFastEMA,MACDSlowEMA,MACDSMA,MACDPrice,0,TradeBar);
   double macd1=iMACD(NULL,0,MACDFastEMA,MACDSlowEMA,MACDSMA,MACDPrice,0,TradeBar+1);   
   double ma0d=iMA(NULL,0,1,0,0,0,TradeBar+1);
   double ma1d=iMA(NULL,0,MAPeriod,0,MAMethod,MAPrice,TradeBar+1+MAShift);
   
   // BULL trend
  if(ma0>=ma1 && ma0d<=ma1 && macd>=0 && macd1<=0)
   {
    return(BULL); 
   }  
   // BEAR trend
  if(ma0<=ma1 && ma0d>=ma1 && macd<=0 && macd1>=0)
   {
    return(BEAR);
   } 
   // Close BULL
  if(macd<=0 && macd1>=0)
   {
    return(CloseBULL); 
   }  
   // Close BEAR
  if(macd>=0 && macd1<=0)
   {
    return(CloseBEAR);
   }    
   
   // flat no trend return 0
   return(0);
}

//--------------------------------------// Dispay Comment //------------------------------------// 

void Display_Info()
{
   Comment(     
            "Balance:  ",AccountBalance(),"\n",
            "Equity:  ",AccountEquity(),"\n",
            "Margin:  ",AccountMargin(),"\n",
            "Free Margin:  ",AccountFreeMargin(),"\n",
            "Lots:  ",Lots,"\n",
            "Symbol: ", Symbol(),"\n",
            "Price:  ",NormalizeDouble(Bid,4),"\n",
            "Pip Spread:  ",MarketInfo("EURUSD",MODE_SPREAD),"\n");
   return(0);
}
//-------------------------------------// Trailing Stops //-------------------------------------//
void TrailPositions()
{
  int Orders = OrdersTotal();
  for (int i=0; i<Orders; i++) 
  {
    if (!(OrderSelect(i, SELECT_BY_POS, MODE_TRADES))) continue;
    if (OrderSymbol() != Symbol()) continue;
    {    
      if (OrderType() == OP_BUY )  
      {
        if (Bid-OrderOpenPrice() > MinProfit*Point) 
        {
          if (OrderStopLoss() < Bid-(TrailingStop+TrailingStep-1)*Point) 
          {
          OrderModify(OrderTicket(), OrderOpenPrice(), Bid-TrailingStop*Point,OrderTakeProfit(), 0, Black);
          }
        }
      }   
      if (OrderType() == OP_SELL)  
      {
        if (OrderOpenPrice()-Ask >MinProfit*Point) 
        {
          if (OrderStopLoss() > Ask+(TrailingStop+TrailingStep-1)*Point || OrderStopLoss() == 0) 
          {
          OrderModify(OrderTicket(), OrderOpenPrice(), Ask+TrailingStop*Point,OrderTakeProfit(), 0, Black);
          }
        }
      }   
    }   
  }  
}